#include <SPI.h>
#include <nRF24L01p.h>
#include <Servo.h>
#include <NewPing.h>

#define CSN PIN_PD5
#define CE PIN_PD6
#define MOTOR_1R PIN_PC0
#define MOTOR_1L PIN_PC1
#define MOTOR_2R PIN_PC2
#define MOTOR_2L PIN_PC3
#define SERVO PIN_PB1
#define trigger PIN_PC4
#define echo PIN_PC5
#define MAX_DIS 400

nRF24L01p receiver(CSN, CE);
NewPing sonar(trigger, echo, MAX_DIS);
Servo servo;
int data[5];

void forward() {
  digitalWrite(MOTOR_1L, HIGH);
  digitalWrite(MOTOR_1R, LOW);
  digitalWrite(MOTOR_2L, HIGH);
  digitalWrite(MOTOR_2R, LOW);
}

void backward() {
  digitalWrite(MOTOR_1L, LOW);
  digitalWrite(MOTOR_1R, HIGH);
  digitalWrite(MOTOR_2L, LOW);
  digitalWrite(MOTOR_2R, HIGH);
}

void right() {
  digitalWrite(MOTOR_1L, HIGH);
  digitalWrite(MOTOR_1R, LOW);
  digitalWrite(MOTOR_2L, LOW);
  digitalWrite(MOTOR_2R, LOW);
}

void left() {
  digitalWrite(MOTOR_1L, LOW);
  digitalWrite(MOTOR_1R, LOW);
  digitalWrite(MOTOR_2L, HIGH);
  digitalWrite(MOTOR_2R, LOW);
}

void stopped() {
  digitalWrite(MOTOR_1L, LOW);
  digitalWrite(MOTOR_1R, LOW);
  digitalWrite(MOTOR_2L, LOW);
  digitalWrite(MOTOR_2R, LOW);
}

void control() {
  if (data[0] == 0) 
    forward();
  
  if (data[1] == 0) 
    backward();
  
  if (data[2] == 0) 
    left();
  
  if (data[3] == 0) 
    right();
  
  if (data[0] == 1 && data[1] == 1 && data[2] == 1 && data[3] == 1) 
    stopped();
  
}
void decide(int this_right, int this_right1, int this_left, int this_left1) {
  if (this_right > this_left) {
    if (this_right < this_right1) {
      backward();
      delay(500);
    }
    right();
    delay(500);

  }
  else {
    if (this_left < this_left1) {
      backward();
      delay(500);
    }
    left();
    delay(500);
  }
}

void autopilot() {
  delay(50);                     // Wait 50ms between pings (about 20 pings/sec). 29ms should be the shortest delay between pings.
  int distance = sonar.ping_cm();
  int limit = 30;
  int dis_right;
  int dis_right1;
  int dis_left;
  int dis_left1;
  forward();
  if (distance <= limit) {
    stopped();

    servo.write(125);
    dis_right1 = sonar.ping_cm();
    delay(100);
    servo.write(160);
    dis_right = sonar.ping_cm();
    delay(500);

    servo.write(55);
    dis_left1 = sonar.ping_cm();
    delay(100);
    servo.write(20);
    dis_left = sonar.ping_cm();
    delay(500);

    servo.write(90);
    delay(500);

    decide(dis_right, dis_right1, dis_left, dis_left1);
  }

}

void setup() {
  delay(150);
  Serial.begin(115200);
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  receiver.channel(90);
  receiver.RXaddress("MOO");
  receiver.init();
  pinMode(MOTOR_1L, OUTPUT);
  pinMode(MOTOR_1R, OUTPUT);
  pinMode(MOTOR_2L, OUTPUT);
  pinMode(MOTOR_2R, OUTPUT);
  servo.attach(SERVO);
  servo.write(90);
}

void loop() {
  if (receiver.available()) {
    receiver.read();
    receiver.rxPL(data, 5);
    Serial.print(data[0]);
    Serial.print(data[1]);
    Serial.print(data[2]);
    Serial.print(data[3]);
    Serial.println(data[4]);
    if (data[4] == 0)
      control();
    else
      autopilot();
  }
}

